@@ -55,15 +55,21 @@ void setup_segfaulter(void)
 int test_mmap(void)
 {
 	comedi_cmd cmd;
-	char *buf;
+	unsigned char *buf;
 	unsigned int chanlist[1];
 	int go;
+	int fails;
 	int total=0;
 	int ret;
-	void *b, *adr;
-	sampl_t *map;
+	void *b;
+	unsigned char *adr;
+	unsigned char *map;
+	unsigned int flags;
+	int i;
 
-	if(!(comedi_get_subdevice_flags(device,subdevice)&SDF_CMD)){
+	flags = comedi_get_subdevice_flags(device,subdevice);
+
+	if(!(flags&SDF_CMD) || flags&SDF_WRITEABLE){
 		printf("not applicable\n");
 		return 0;
 	}
@@ -84,7 +90,7 @@ int test_mmap(void)
 	}
 
 	/* test readability */
-	for(adr=map;adr<(void *)map+MAPLEN;adr+=PAGE_SIZE){
+	for(adr=map;adr<map+MAPLEN;adr+=PAGE_SIZE){
 		ret=test_segfault(adr);
 		if(ret){
 			printf("E: %p failed\n",adr);
@@ -122,15 +128,24 @@ int test_mmap(void)
 			if(verbose) printf("read %d %d\n",ret,total);
 		}
 	}
-	if(memcmp(buf,map,total)){
-		printf("E: mmap compare failed\n");
-	}else{
-		printf("compare ok\n");
+
+	fails = 0;
+	for(i=0;i<total;i++){
+		if(buf[i]!=map[i]){
+			if(fails==0)printf("E: mmap compare failed\n");
+			printf("offset %d (read=%02x mmap=%02x)\n",i,
+				buf[i], map[i]);
+			go = 0;
+			fails++;
+			if(fails>10)break;
+		}
 	}
+	if(fails==0) printf("compare ok\n");
+
 	munmap(map,MAPLEN);
 
 	/* test if area is really unmapped */
-	for(adr=map;adr<(void *)map+MAPLEN;adr+=PAGE_SIZE){
+	for(adr=map;adr<map+MAPLEN;adr+=PAGE_SIZE){
 		ret=test_segfault(adr);
 		if(ret){
 			printf("%p segfaulted (ok)\n",adr);
